use anyhow::{anyhow, Result};
use reqwest::{header, Client};
use std::path::PathBuf;
use structopt::StructOpt;

mod cmd;
mod gitea;

#[derive(StructOpt, Debug)]
pub(crate) struct Common {
    /// The gitea server to connect to
    #[structopt(short, long, env = "GITEA_SERVER")]
    server: String,
    /// The gitea token to authenticate with
    #[structopt(long, env = "GITEA_TOKEN")]
    token: String,
    /// The gitea user to authenticate as
    #[structopt(short, long, env = "GITEA_AUTH_USER")]
    auth_user: String,
    /// The owner of the gitea repo
    #[structopt(short, long, env = "GITEA_OWNER")]
    owner: String,
    /// The gitea repo to operate on
    #[structopt(short, long, env = "GITEA_REPO")]
    repo: String,
    /// The version tag to operate on
    #[structopt(short, long)]
    tag: Option<String>,
}

// Name your user agent after your app?
static APP_USER_AGENT: &str = concat!(env!("CARGO_PKG_NAME"), "/", env!("CARGO_PKG_VERSION"));

pub(crate) fn client(c: &Common) -> Result<Client> {
    let mut headers = header::HeaderMap::new();
    let auth = format!("token {}", &c.token);
    let auth = auth.as_str();
    headers.insert(header::AUTHORIZATION, header::HeaderValue::from_str(auth)?);
    Ok(Client::builder()
        .user_agent(APP_USER_AGENT)
        .default_headers(headers)
        .build()?)
}

#[derive(StructOpt, Debug)]
pub(crate) struct ReleaseMeta {
    /// Release name
    #[structopt(short, long, default_value = "")]
    name: String,
    /// Draft release
    #[structopt(long)]
    draft: bool,
    /// Pre-release (not suitable for production)
    #[structopt(short, long)]
    pre_release: bool,
}

#[derive(StructOpt, Debug)]
#[structopt(about = "Gitea release assistant")]
pub(crate) enum Cmd {
    /// Delete a given release from Gitea
    Delete {
        #[structopt(flatten)]
        common: Common,
    },
    /// Downloads release artifacts
    Download {
        #[structopt(flatten)]
        common: Common,
        /// Folder to download release artifacts to
        #[structopt(short, long)]
        fname: Option<PathBuf>,
    },
    /// Edits a release's description, name and other flags
    Edit {
        #[structopt(flatten)]
        common: Common,
        /// Release description
        #[structopt(short, long, default_value = "")]
        description: String,
        #[structopt(flatten)]
        release_meta: ReleaseMeta,
    },
    /// Gets release info
    Info {
        #[structopt(flatten)]
        common: Common,
        #[structopt(long, short)]
        json: bool,
    },
    /// Create a new tag and release on Gitea
    Release {
        #[structopt(flatten)]
        common: Common,
        /// Changelog file to read from to create the release description
        #[structopt(short, long, default_value = "./CHANGELOG.md")]
        changelog: PathBuf,
        #[structopt(flatten)]
        release_meta: ReleaseMeta,
    },
    /// Uploads release artifacts to Gitea
    Upload {
        #[structopt(flatten)]
        common: Common,
        /// The name of the file
        #[structopt(short, long, default_value = "")]
        name: String,
        /// The description of the file
        #[structopt(short, long, default_value = "")]
        label: String,
        /// The location of the file on the disk
        #[structopt(short, long)]
        fname: PathBuf,
        /// Replace existing release artifacts?
        #[structopt(long)]
        replace: bool,
    },
}

#[tokio::main]
async fn main() -> Result<()> {
    let _ = kankyo::init();
    let cmd = Cmd::from_args();

    match cmd {
        Cmd::Delete { common } => cmd::delete::run(common).await,
        Cmd::Download { common, fname } => cmd::download::run(common, fname).await,
        Cmd::Info { common, json } => cmd::info::run(common, json).await,

        _ => Err(anyhow!("not implemented yet")),
    }
}
